﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.Mvc;
using Kreta.BusinessLogic.HelperClasses;
using Kreta.BusinessLogic.HelperClasses.ImportCo;
using Kreta.BusinessLogic.Helpers;
using Kreta.BusinessLogic.Helpers.ImportExport;
using Kreta.BusinessLogic.Security;
using Kreta.Enums.ManualEnums;
using Kreta.Enums.ManualEnums.ImportExport;
using Kreta.Resources;
using Kreta.Web.Areas.ImportExport.Models.CommonModels;
using Kreta.Web.Areas.ImportExport.Models.TanuloBesorolasImportExport;
using Kreta.Web.Classes;
using Kreta.Web.Helpers;
using Kreta.Web.Models.EditorTemplates;
using Kreta.Web.Security;

namespace Kreta.Web.Areas.ImportExport.Controllers
{
    [MvcRoleClaimsAuthorize(true)]
    [MvcRolePackageDenyAuthorize(KretaClaimPackages.IsOnlyAlkalmozott.ClaimValue)]
    [MvcRolePackageAuthorize(TanevEnum.AktEsKovTanev, KretaClaimPackages.Adminisztrator.ClaimValue)]
    public class TanuloBesorolasImportExportController : BaseImportExportController
    {
        #region Fields

        private const string TovabbUrl = @"~/Tanulo/Tanulo";

        #endregion Fields

        #region Properties

        public static string GridName => "TanuloBesorolasGrid";
        public static string GridRowTemplateName => "TanuloBesorolasGridRowTemplate";

        public static string JavascriptHelperClassName => "TanuloBesorolasImportExportHelper";

        public static string ConrtollerName => "TanuloBesorolasImportExport";

        #endregion Properties

        #region Constructors

        public TanuloBesorolasImportExportController(IUploadFileValidator uploadFileValidator)
        {
            UploadFileValidator = uploadFileValidator ?? throw new ArgumentNullException(nameof(uploadFileValidator));
        }

        #endregion Constructors

        public ActionResult Index()
        {
            var helper = new TanevHelper(ConnectionTypeExtensions.GetSessionConnectionType());
            TanevCO tanevCo = helper.GetTanevInfo();
            DateTime besorolasDatumMinDate = tanevCo.KezdoNap;
            DateTime besorolasDatumMaxDate = tanevCo.UtolsoNap;

            var today = DateTime.Today;
            DateTime besorolasDatumValue;
            if (besorolasDatumMinDate > today)
            {
                besorolasDatumValue = besorolasDatumMinDate;
            }
            else if (besorolasDatumMaxDate < today)
            {
                besorolasDatumValue = besorolasDatumMaxDate;
            }
            else
            {
                besorolasDatumValue = today;
            }

            var model = new TanuloBesorolasImportExportModel
            {
                MaxAllowedFileSizeInBytes = UploadFileValidator.Configuration.ImportMaxAllowedFileSizeInBytes,
                AllowedFileExtensions = UploadFileValidator.Configuration.ImportAllowedFileExtensions,
                BesorolasDatumValue = besorolasDatumValue,
                BesorolasDatumMinDate = besorolasDatumMinDate,
                BesorolasDatumMaxDate = besorolasDatumMaxDate,
                IsSzakkepzoIntezmeny = ClaimData.IsSzakkepzoIntezmeny,
            };

            return View(model);
        }

        #region Sablon

        public ActionResult DownloadTemplate()
        {
            MemoryStream memoryStream = new TanuloBesorolasImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetTemplate();
            return new FileStreamResult(memoryStream, Core.Constants.ContentTypes.Xlsx) { FileDownloadName = ImportExportTanuloBesorolasResource.ImportSablonFileName };
        }

        #endregion Sablon

        #region Export

        [HttpPost]
        [MvcValidateAjaxAntiForgeryToken]
        public string Export()
        {
            MemoryStream memoryStream = new TanuloBesorolasImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetExport();
            return GetExportFileString(memoryStream);
        }

        public string NemImportalhatoSorokExport()
        {
            MemoryStream memoryStream = new TanuloBesorolasImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetNemImportalhatoSorokExport();
            return GetExportFileString(memoryStream);
        }

        #endregion Export

        #region Import

        public JsonResult UploadFile(HttpPostedFileBase importFile, DateTime? besorolasDatum, bool? isJuttatasUpdate, DateTime? kisorolasDatum)
        {
            UploadFileValidator.ValidateImportFile(importFile);

            var errorResult = new List<string>();

            if (besorolasDatum == null)
            {
                errorResult.Add(ImportExportTanuloBesorolasResource.RequiredBesorolasDatum);
            }
            else
            {
                var helper = new TanevHelper(ConnectionTypeExtensions.GetSessionConnectionType());
                TanevCO tanevCo = helper.GetTanevInfo();
                DateTime besorolasDatumMinDate = tanevCo.KezdoNap;
                DateTime besorolasDatumMaxDate = tanevCo.UtolsoNap;

                if (besorolasDatum < besorolasDatumMinDate)
                {
                    errorResult.Add(string.Format(ImportExportCommonResource.InvalidKorabbiDatum, ImportExportTanuloBesorolasResource.BesorolasDatum, besorolasDatum.Value.ToString(Core.Constants.ToStringPattern.HungarianDate), ImportExportCommonResource.ElsoTanitasiNap.ToLowerInvariant(), besorolasDatumMinDate.ToString(Core.Constants.ToStringPattern.HungarianDate)));
                }
                else if (besorolasDatum > besorolasDatumMaxDate)
                {
                    errorResult.Add(string.Format(ImportExportCommonResource.InvalidKesobbiDatum, ImportExportTanuloBesorolasResource.BesorolasDatum, besorolasDatum.Value.ToString(Core.Constants.ToStringPattern.HungarianDate), ImportExportCommonResource.UtolsoTanitasiNap.ToLowerInvariant(), besorolasDatumMaxDate.ToString(Core.Constants.ToStringPattern.HungarianDate)));
                }

                if (kisorolasDatum.HasValue && besorolasDatum >= kisorolasDatum)
                {
                    errorResult.Add(string.Format(ImportExportCommonResource.InvalidKesobbiDatum, ImportExportTanuloBesorolasResource.BesorolasDatum, besorolasDatum.Value.ToString(Core.Constants.ToStringPattern.HungarianDate), ImportExportTanuloBesorolasResource.KisorolasDatum.ToLowerInvariant(), kisorolasDatum.Value.ToString(Core.Constants.ToStringPattern.HungarianDate)));
                }
            }

            Dictionary<int, string> importHeaderList = TanuloBesorolasImportExportHelper.ImportHeaderList;
            List<List<string>> importData = ImportExportHelper.GetImportDataFromExcel(importFile.InputStream, importHeaderList, out List<string> fileErrorResultList);
            errorResult.AddRange(fileErrorResultList);

            if (errorResult.Count > 0)
            {
                JsonResult errorJson = ThrowCustomError(errorResult);
                return errorJson;
            }

            var isIntezmenySzakkepzoJuttatas = new IntezmenyHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetIntezmenyiAdatok().IsSzakkepzoJuttatas;
            //NOTE: besorolasDatum.Value nem lehet null a korábbi vizsgálatok miatt!
            new TanuloBesorolasImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).SetImportCo(importData, besorolasDatum.Value, isJuttatasUpdate, isIntezmenySzakkepzoJuttatas, kisorolasDatum, ClaimData.IsSzakkepzoIntezmeny);

            return Json(null);
        }

        public ActionResult OpenImportPreviewPopup()
        {
            TanuloBesorolasImportCo importCo = new TanuloBesorolasImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).ImportCo;

            var importInfoModel = new TanuloBesorolasImportInfoModel
            {
                ImportInfoSummaryList = new ImportInfoSummaryListModel
                {
                    ImportInfoSummaryModelList = new List<ImportInfoSummaryModel>
                    {
                        GetImportInfoSummary(importCo)
                    }
                },
                MainImportItemList = new TanuloBesorolasImportItemListModel
                {
                    ImportItemModelList = GetImportItemModelList(importCo)
                },
                NemImportalhatoItemList = new TanuloBesorolasImportNemImportalhatoItemListModel
                {
                    NemImportalhatoItemModelList = GetNemImportalhatoItemModelList(importCo.NemImportalhatoItemList)
                }
            };

            var popUpModel = new PopUpModel(importInfoModel, ImportPreviewPopupName);
            popUpModel.AddCancelBtn(popUpModel, "KretaWindowHelper.destroyAllWindow");
            popUpModel.AddBtn(popUpModel, "ImportBtnOk", CommonResource.Importalas, $"{JavascriptHelperClassName}.import", "BtnOk", setDisabledAfterClick: true);
            popUpModel.AddBtn(popUpModel, "NemImportalhatoSorokExportBtnOk", ImportExportCommonResource.NemImportalhatoSorokExportalasa, $"{JavascriptHelperClassName}.nemImportalhatoSorokExport", "BtnOk");
            return PartialView(Constants.General.PopupView, popUpModel);
        }

        private static ImportInfoSummaryModel GetImportInfoSummary(TanuloBesorolasImportCo importCo)
        {
            List<TanuloCsoportItemCo> coList = importCo.TanuloCsoportCoList;
            List<TanuloBesorolasImportJsonItemCo> mainImportJsonItemList = importCo.MainImportJsonItemList;
            int insertRowCount = mainImportJsonItemList?.Count(x => x.Operation == (int)ImportItemOperationEnum.Insert) ?? 0;

            int invalidRowCount = importCo.NemImportalhatoItemList.Count;
            var importInfoSummary = new ImportInfoSummaryModel
            {
                ImportName = ImportExportTanuloBesorolasResource.ImportDefaultSheetName,
                InsertRowCount = insertRowCount.ToString(),
                UpdateRowCount = "-",
                DeleteRowCount = "-",
                UnchangedRowCount = coList.Count.ToString(),
                InvalidRowCount = invalidRowCount.ToString()
            };
            return importInfoSummary;
        }

        private List<TanuloBesorolasImportItemModel> GetImportItemModelList(TanuloBesorolasImportCo importCo)
        {
            List<TanuloCsoportItemCo> coList = importCo.TanuloCsoportCoList;
            var importItemModelList = new List<TanuloBesorolasImportItemModel>();
            List<TanuloBesorolasImportJsonItemCo> mainImportJsonItemList = importCo.MainImportJsonItemList;
            if (mainImportJsonItemList == null)
            {
                if (!(coList.Count > 0))
                {
                    return importItemModelList;
                }

                mainImportJsonItemList = new List<TanuloBesorolasImportJsonItemCo>();
            }

            //NOTE: Hozzáadjuk azokat a az elemeket, amik az excel-ből jöttek!
            foreach (TanuloBesorolasImportJsonItemCo importJsonItem in mainImportJsonItemList)
            {
                importItemModelList.Add(new TanuloBesorolasImportItemModel
                {
                    TanuloElotag = importJsonItem.TanuloElotag ?? string.Empty,
                    TanuloVezeteknev = importJsonItem.TanuloVezeteknev,
                    TanuloKeresztnev = importJsonItem.TanuloKeresztnev,
                    TanuloOktatasiAzonosito = importJsonItem.TanuloOktatasiAzonosito,
                    OsztalyCsoportNev = importJsonItem.OsztalyCsoportNev,
                    Operation = importJsonItem.Operation
                });
            }

            //NOTE: Hozzáadjuk azokat a az elemeket, amik az adatbázisból jöttek!
            foreach (TanuloCsoportItemCo co in coList)
            {
                if (!mainImportJsonItemList.Select(x => x.Id).Contains(co.Id))
                {
                    importItemModelList.Add(new TanuloBesorolasImportItemModel
                    {
                        TanuloElotag = co.FelhasznaloElotag ?? string.Empty,
                        TanuloVezeteknev = co.FelhasznaloVezeteknev,
                        TanuloKeresztnev = co.FelhasznaloKeresztnev,
                        TanuloOktatasiAzonosito = co.FelhasznaloOktatasiAzonosito,
                        OsztalyCsoportNev = co.OsztalyCsoportNev,
                        Operation = (int)ImportItemOperationEnum.Default
                    });
                }
            }

            return importItemModelList.OrderBy(x => x.TanuloVezeteknev).ThenBy(x => x.TanuloKeresztnev).ThenBy(x => x.OsztalyCsoportNev).ThenByDescending(x => x.OperationText).ToList();
        }

        private List<TanuloBesorolasImportNemImportalhatoItemModel> GetNemImportalhatoItemModelList(List<TanuloBesorolasImportItemCo> nemImportalhatoItemList)
        {
            var nemImportalhatoItemModelList = new List<TanuloBesorolasImportNemImportalhatoItemModel>();
            foreach (TanuloBesorolasImportItemCo nemImportalhatoItem in nemImportalhatoItemList)
            {
                nemImportalhatoItemModelList.Add(new TanuloBesorolasImportNemImportalhatoItemModel
                {
                    TanuloNeveImportData = nemImportalhatoItem.TanuloNeveImportData,
                    TanuloOktatasiAzonositoImportData = nemImportalhatoItem.TanuloOktatasiAzonositoImportData,
                    OsztalyCsoportImportData = nemImportalhatoItem.OsztalyCsoportImportData,

                    ErrorList = string.Join(Environment.NewLine, nemImportalhatoItem.ErrorList)
                });
            }

            return nemImportalhatoItemModelList.OrderBy(x => x.TanuloNeveImportData).ThenBy(x => x.OsztalyCsoportImportData).ToList();
        }

        public ActionResult Import()
        {
            if (ModelState.IsValid)
            {
                var helper = new TanuloBesorolasImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType());
                TanuloBesorolasImportCo importCo = helper.ImportCo;
                helper.Import(new { ImportJson = importCo }, importCo.BesorolasDatum);

                return Content(Url.Content(TovabbUrl));
            }

            return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
        }

        #endregion Import
    }
}
